@@ -10,12 +10,15 @@ import collections
 import ctypes
 import numbers
 
-from .api import library, MagickPixelPacket, PointInfo, AffineMatrix
+from .api import (AffineMatrix, MagickPixelPacket, PixelInfo, PointInfo,
+                  library)
 from .color import Color
 from .compat import binary, string_type, text, text_type, xrange
-from .image import Image, COMPOSITE_OPERATORS
-from .resource import Resource
 from .exceptions import WandLibraryVersionError
+from .image import BaseImage, COMPOSITE_OPERATORS
+from .sequence import SingleImage
+from .resource import Resource
+from .version import MAGICK_VERSION_NUMBER
 
 __all__ = ('CLIP_PATH_UNITS', 'FILL_RULE_TYPES', 'FONT_METRICS_ATTRIBUTES',
            'GRAVITY_TYPES', 'LINE_CAP_TYPES', 'LINE_JOIN_TYPES',
@@ -181,15 +184,6 @@ class Drawing(Resource):
                 wand = library.CloneDrawingWand(drawing.resource)
             self.resource = wand
 
-    def clone(self):
-        """Copies a drawing object.
-
-        :returns: a duplication
-        :rtype: :class:`Drawing`
-
-        """
-        return type(self)(drawing=self)
-
     @property
     def border_color(self):
         """(:class:`~wand.color.Color`) the current border color. It also can
@@ -201,9 +195,14 @@ class Drawing(Resource):
         """
         pixelwand = library.NewPixelWand()
         library.DrawGetBorderColor(self.resource, pixelwand)
-        size = ctypes.sizeof(MagickPixelPacket)
+        if MAGICK_VERSION_NUMBER < 0x700:
+            pixel_structure = MagickPixelPacket
+        else:
+            pixel_structure = PixelInfo
+        size = ctypes.sizeof(pixel_structure)
         buffer = ctypes.create_string_buffer(size)
         library.PixelGetMagickColor(pixelwand, buffer)
+        pixelwand = library.DestroyPixelWand(pixelwand)
         return Color(raw=buffer)
 
     @border_color.setter
@@ -232,9 +231,7 @@ class Drawing(Resource):
     def clip_path(self, path):
         if not isinstance(path, string_type):
             raise TypeError('expected a string, not ' + repr(path))
-        okay = library.DrawSetClipPath(self.resource, binary(path))
-        if okay == 0:
-            raise ValueError('Clip path not understood')
+        library.DrawSetClipPath(self.resource, binary(path))
 
     @property
     def clip_rule(self):
@@ -276,6 +273,70 @@ class Drawing(Resource):
         library.DrawSetClipUnits(self.resource,
                                  CLIP_PATH_UNITS.index(clip_unit))
 
+    @property
+    def fill_color(self):
+        """(:class:`~wand.color.Color`) The current color to fill.
+        It also can be set.
+
+        """
+        pixel = library.NewPixelWand()
+        library.DrawGetFillColor(self.resource, pixel)
+        if MAGICK_VERSION_NUMBER < 0x700:
+            pixel_structure = MagickPixelPacket
+        else:
+            pixel_structure = PixelInfo
+        size = ctypes.sizeof(pixel_structure)
+        buffer = ctypes.create_string_buffer(size)
+        library.PixelGetMagickColor(pixel, buffer)
+        pixel = library.DestroyPixelWand(pixel)
+        return Color(raw=buffer)
+
+    @fill_color.setter
+    def fill_color(self, color):
+        if not isinstance(color, Color):
+            raise TypeError('color must be a wand.color.Color object, not ' +
+                            repr(color))
+        with color:
+            library.DrawSetFillColor(self.resource, color.resource)
+
+    @property
+    def fill_opacity(self):
+        """(:class:`~numbers.Real`) The current fill opacity.
+        It also can be set.
+
+        .. versionadded:: 0.4.0
+        """
+        return library.DrawGetFillOpacity(self.resource)
+
+    @fill_opacity.setter
+    def fill_opacity(self, opacity):
+        if not isinstance(opacity, numbers.Real):
+            raise TypeError('opacity must be a double, not ' +
+                            repr(opacity))
+        library.DrawSetFillOpacity(self.resource, opacity)
+
+    @property
+    def fill_rule(self):
+        """(:class:`basestring`) The current fill rule. It can also be set.
+        It's a string value from :const:`FILL_RULE_TYPES` list.
+
+        .. versionadded:: 0.4.0
+        """
+        fill_rule_index = library.DrawGetFillRule(self.resource)
+        if fill_rule_index not in FILL_RULE_TYPES:
+            self.raise_exception()
+        return text(FILL_RULE_TYPES[fill_rule_index])
+
+    @fill_rule.setter
+    def fill_rule(self, fill_rule):
+        if not isinstance(fill_rule, string_type):
+            raise TypeError('expected a string, not ' + repr(fill_rule))
+        elif fill_rule not in FILL_RULE_TYPES:
+            raise ValueError('expected a string from FILE_RULE_TYPES, not' +
+                             repr(fill_rule))
+        library.DrawSetFillRule(self.resource,
+                                FILL_RULE_TYPES.index(fill_rule))
+
     @property
     def font(self):
         """(:class:`basestring`) The current font name.  It also can be set.
@@ -350,7 +411,9 @@ class Drawing(Resource):
 
     @property
     def font_stretch(self):
-        """(:class:`basestring`) The current font family. It also can be set.
+        """(:class:`basestring`) The current font stretch variation.
+        It also can be set, but will only apply if the font-family or encoder
+        supports the stretch type.
 
         .. versionadded:: 0.4.0
         """
@@ -369,7 +432,9 @@ class Drawing(Resource):
 
     @property
     def font_style(self):
-        """(:class:`basestring`) The current font style. It also can be set.
+        """(:class:`basestring`) The current font style.
+        It also can be set, but will only apply if the font-family
+        supports the style.
 
         .. versionadded:: 0.4.0
         """
@@ -401,65 +466,6 @@ class Drawing(Resource):
             raise TypeError('expected a integral, not ' + repr(weight))
         library.DrawSetFontWeight(self.resource, weight)
 
-    @property
-    def fill_color(self):
-        """(:class:`~wand.color.Color`) The current color to fill.
-        It also can be set.
-
-        """
-        pixel = library.NewPixelWand()
-        library.DrawGetFillColor(self.resource, pixel)
-        size = ctypes.sizeof(MagickPixelPacket)
-        buffer = ctypes.create_string_buffer(size)
-        library.PixelGetMagickColor(pixel, buffer)
-        return Color(raw=buffer)
-
-    @fill_color.setter
-    def fill_color(self, color):
-        if not isinstance(color, Color):
-            raise TypeError('color must be a wand.color.Color object, not ' +
-                            repr(color))
-        with color:
-            library.DrawSetFillColor(self.resource, color.resource)
-
-    @property
-    def fill_opacity(self):
-        """(:class:`~numbers.Real`) The current fill opacity.
-        It also can be set.
-
-        .. versionadded:: 0.4.0
-        """
-        return library.DrawGetFillOpacity(self.resource)
-
-    @fill_opacity.setter
-    def fill_opacity(self, opacity):
-        if not isinstance(opacity, numbers.Real):
-            raise TypeError('opacity must be a double, not ' +
-                            repr(opacity))
-        library.DrawSetFillOpacity(self.resource, opacity)
-
-    @property
-    def fill_rule(self):
-        """(:class:`basestring`) The current fill rule. It can also be set.
-        It's a string value from :const:`FILL_RULE_TYPES` list.
-
-        .. versionadded:: 0.4.0
-        """
-        fill_rule_index = library.DrawGetFillRule(self.resource)
-        if fill_rule_index not in FILL_RULE_TYPES:
-            self.raise_exception()
-        return text(FILL_RULE_TYPES[fill_rule_index])
-
-    @fill_rule.setter
-    def fill_rule(self, fill_rule):
-        if not isinstance(fill_rule, string_type):
-            raise TypeError('expected a string, not ' + repr(fill_rule))
-        elif fill_rule not in FILL_RULE_TYPES:
-            raise ValueError('expected a string from FILE_RULE_TYPES, not' +
-                             repr(fill_rule))
-        library.DrawSetFillRule(self.resource,
-                                FILL_RULE_TYPES.index(fill_rule))
-
     @property
     def opacity(self):
         """(:class:`~numbers.Real`) returns the opacity used when drawing with
@@ -472,6 +478,27 @@ class Drawing(Resource):
         """
         return library.DrawGetOpacity(self.resource)
 
+    @property
+    def gravity(self):
+        """(:class:`basestring`) The text placement gravity used when
+        annotating with text.  It's a string from :const:`GRAVITY_TYPES`
+        list.  It also can be set.
+
+        """
+        gravity_index = library.DrawGetGravity(self.resource)
+        if not gravity_index:
+            self.raise_exception()
+        return text(GRAVITY_TYPES[gravity_index])
+
+    @gravity.setter
+    def gravity(self, value):
+        if not isinstance(value, string_type):
+            raise TypeError('expected a string, not ' + repr(value))
+        elif value not in GRAVITY_TYPES:
+            raise ValueError('expected a string from GRAVITY_TYPES, not ' +
+                             repr(value))
+        library.DrawSetGravity(self.resource, GRAVITY_TYPES.index(value))
+
     @opacity.setter
     def opacity(self, opaque):
         library.DrawSetOpacity(self.resource, ctypes.c_double(opaque))
@@ -505,7 +532,11 @@ class Drawing(Resource):
         """
         pixel = library.NewPixelWand()
         library.DrawGetStrokeColor(self.resource, pixel)
-        size = ctypes.sizeof(MagickPixelPacket)
+        if MAGICK_VERSION_NUMBER < 0x700:
+            pixel_structure = MagickPixelPacket
+        else:
+            pixel_structure = PixelInfo
+        size = ctypes.sizeof(pixel_structure)
         buffer = ctypes.create_string_buffer(size)
         library.PixelGetMagickColor(pixel, buffer)
         return Color(raw=buffer)
@@ -824,9 +855,14 @@ class Drawing(Resource):
         """
         pixel = library.NewPixelWand()
         library.DrawGetTextUnderColor(self.resource, pixel)
-        size = ctypes.sizeof(MagickPixelPacket)
+        if MAGICK_VERSION_NUMBER < 0x700:
+            pixel_structure = MagickPixelPacket
+        else:
+            pixel_structure = PixelInfo
+        size = ctypes.sizeof(pixel_structure)
         buffer = ctypes.create_string_buffer(size)
         library.PixelGetMagickColor(pixel, buffer)
+        pixel = library.DestroyPixelWand(pixel)
         return Color(raw=buffer)
 
     @text_under_color.setter
@@ -871,53 +907,6 @@ class Drawing(Resource):
             if okay == 0:
                 raise ValueError("Vector graphic not understood.")
 
-    @property
-    def gravity(self):
-        """(:class:`basestring`) The text placement gravity used when
-        annotating with text.  It's a string from :const:`GRAVITY_TYPES`
-        list.  It also can be set.
-
-        """
-        gravity_index = library.DrawGetGravity(self.resource)
-        if not gravity_index:
-            self.raise_exception()
-        return text(GRAVITY_TYPES[gravity_index])
-
-    @gravity.setter
-    def gravity(self, value):
-        if not isinstance(value, string_type):
-            raise TypeError('expected a string, not ' + repr(value))
-        elif value not in GRAVITY_TYPES:
-            raise ValueError('expected a string from GRAVITY_TYPES, not ' +
-                             repr(value))
-        library.DrawSetGravity(self.resource, GRAVITY_TYPES.index(value))
-
-    def clear(self):
-        library.ClearDrawingWand(self.resource)
-
-    def draw(self, image):
-        """Renders the current drawing into the ``image``.  You can simply
-        call :class:`Drawing` instance rather than calling this method.
-        That means the following code which calls :class:`Drawing` object
-        itself::
-
-            drawing(image)
-
-        is equivalent to the following code which calls :meth:`draw()` method::
-
-            drawing.draw(image)
-
-        :param image: the image to be drawn
-        :type image: :class:`~wand.image.Image`
-
-        """
-        if not isinstance(image, Image):
-            raise TypeError('image must be a wand.image.Image instance, not ' +
-                            repr(image))
-        res = library.MagickDrawImage(image.wand, self.resource)
-        if not res:
-            self.raise_exception()
-
     def affine(self, matrix):
         """Adjusts the current affine transformation matrix with the specified
         affine transformation matrix. Note that the current affine transform is
@@ -945,7 +934,46 @@ class Drawing(Resource):
         amx = AffineMatrix(sx=matrix[0], rx=matrix[1],
                            ry=matrix[2], sy=matrix[3],
                            tx=matrix[4], ty=matrix[5])
-        library.DrawAffine(self.resource, amx)
+        library.DrawAffine(self.resource, ctypes.byref(amx))
+
+    def alpha(self, x=None, y=None, paint_method='undefined'):
+        """Paints on the image's opacity channel in order to set effected pixels
+        to transparent.
+
+         To influence the opacity of pixels. The available methods are:
+
+        - ``'undefined'``
+        - ``'point'``
+        - ``'replace'``
+        - ``'floodfill'``
+        - ``'filltoborder'``
+        - ``'reset'``
+
+        .. note::
+
+            This method replaces :meth:`matte()` in ImageMagick version 7.
+            An :class:`AttributeError` will be raised if attempting
+            to call on a library without ``DrawAlpha`` support.
+
+        .. versionadded:: 0.5.0
+
+        """
+        if library.DrawAlpha is None:
+            raise AttributeError(
+                'Method added with ImageMagick version 7. ' +
+                'Please use `wand.drawing.Drawing.matte()\' instead.'
+            )
+        if x is None or y is None:
+            raise TypeError('Both x & y coordinates need to be defined')
+        if not isinstance(paint_method, string_type):
+            raise TypeError('expected a string, not ' + repr(paint_method))
+        elif paint_method not in PAINT_METHOD_TYPES:
+            raise ValueError(
+                'expected a string from PAINT_METHOD_TYPES, not ' +
+                repr(paint_method)
+            )
+        library.DrawAlpha(self.resource, float(x), float(y),
+                          PAINT_METHOD_TYPES.index(paint_method))
 
     def arc(self, start, end, degree):
         """Draws a arc using the current :attr:`stroke_color`,
@@ -972,6 +1000,37 @@ class Drawing(Resource):
                         float(end_x), float(end_y),
                         float(degree_start), float(degree_end))
 
+    def bezier(self, points=None):
+        """Draws a bezier curve through a set of points on the image, using
+        the specified array of coordinates.
+
+        At least four points should be given to complete a bezier path.
+        The first & forth point being the start & end point, and the second
+        & third point controlling the direction & curve.
+
+        Example bezier on ``image`` ::
+
+            with Drawing() as draw:
+                points = [(40,10), # Start point
+                          (20,50), # First control
+                          (90,10), # Second control
+                          (70,40)] # End point
+                draw.stroke_color = Color('#000')
+                draw.fill_color = Color('#fff')
+                draw.bezier(points)
+                draw.draw(image)
+
+        :param points: list of x,y tuples
+        :type points: :class:`list`
+
+        .. versionadded:: 0.4.0
+
+        """
+
+        (points_l, points_p) = _list_to_point_info(points)
+        library.DrawBezier(self.resource, points_l,
+                           ctypes.cast(points_p, ctypes.POINTER(PointInfo)))
+
     def circle(self, origin, perimeter):
         """Draws a circle from ``origin`` to ``perimeter``
 
@@ -991,6 +1050,18 @@ class Drawing(Resource):
                            float(origin_x), float(origin_y),  # origin
                            float(perimeter_x), float(perimeter_y))  # perimeter
 
+    def clear(self):
+        library.ClearDrawingWand(self.resource)
+
+    def clone(self):
+        """Copies a drawing object.
+
+        :returns: a duplication
+        :rtype: :class:`Drawing`
+
+        """
+        return type(self)(drawing=self)
+
     def color(self, x=None, y=None, paint_method='undefined'):
         """Draws a color on the image using current fill color, starting
         at specified position & method.
@@ -1078,6 +1149,35 @@ class Drawing(Resource):
         if okay == 0:
             self.raise_exception()
 
+    def draw(self, image):
+        """Renders the current drawing into the ``image``.  You can simply
+        call :class:`Drawing` instance rather than calling this method.
+        That means the following code which calls :class:`Drawing` object
+        itself::
+
+            drawing(image)
+
+        is equivalent to the following code which calls :meth:`draw()` method::
+
+            drawing.draw(image)
+
+        :param image: the image to be drawn
+        :type image: :class:`~wand.image.BaseImage`
+
+        """
+        if not isinstance(image, BaseImage):
+            raise TypeError('image must be a wand.image.BaseImage instance,'
+                            ' not ' + repr(image))
+        if isinstance(image, SingleImage):
+            previous = library.MagickGetIteratorIndex(image.container.wand)
+            library.MagickSetIteratorIndex(image.container.wand, image.index)
+            res = library.MagickDrawImage(image.container.wand, self.resource)
+            library.MagickSetIteratorIndex(image.container.wand, previous)
+        else:
+            res = library.MagickDrawImage(image.wand, self.resource)
+        if not res:
+            self.raise_exception()
+
     def ellipse(self, origin, radius, rotation=(0, 360)):
         """Draws a ellipse at ``origin`` with independent x & y ``radius``.
         Ellipse can be partial by setting start & end ``rotation``.
@@ -1104,6 +1204,35 @@ class Drawing(Resource):
                             float(radius_x), float(radius_y),  # radius
                             float(rotation_start), float(rotation_end))
 
+    def get_font_metrics(self, image, text, multiline=False):
+        """Queries font metrics from the given ``text``.
+
+        :param image: the image to be drawn
+        :type image: :class:`~wand.image.BaseImage`
+        :param text: the text string for get font metrics.
+        :type text: :class:`basestring`
+        :param multiline: text is multiline or not
+        :type multiline: `boolean`
+
+        """
+        if not isinstance(image, BaseImage):
+            raise TypeError('image must be a wand.image.BaseImage instance,'
+                            ' not ' + repr(image))
+        if not isinstance(text, string_type):
+            raise TypeError('text must be a string, not ' + repr(text))
+        if multiline:
+            font_metrics_f = library.MagickQueryMultilineFontMetrics
+        else:
+            font_metrics_f = library.MagickQueryFontMetrics
+        if isinstance(text, text_type):
+            if self.text_encoding:
+                text = text.encode(self.text_encoding)
+            else:
+                text = binary(text)
+        result = font_metrics_f(image.wand, self.resource, text)
+        args = (result[i] for i in xrange(13))
+        return FontMetrics(*args)
+
     def line(self, start, end):
         """Draws a line ``start`` to ``end``.
 
@@ -1134,9 +1263,20 @@ class Drawing(Resource):
         - ``'filltoborder'``
         - ``'reset'``
 
+        .. note::
+
+            This method has been replace by :meth:`alpha()` in ImageMagick
+            version 7. An :class:`AttributeError` will be raised if attempting
+            to call on a library without ``DrawMatte`` support.
+
         .. versionadded:: 0.4.0
 
         """
+        if library.DrawMatte is None:
+            raise AttributeError(
+                'Method removed from ImageMagick version. ' +
+                'Please use `wand.drawing.Drawing.alpha()\' instead.'
+            )
         if x is None or y is None:
             raise TypeError('Both x & y coordinates need to be defined')
         if not isinstance(paint_method, string_type):
@@ -1330,31 +1470,6 @@ class Drawing(Resource):
         library.DrawPathFinish(self.resource)
         return self
 
-    def path_line(self, to=None, relative=False):
-        """Draws a line path from the current point to the given ``to``
-        coordinate. The ``to`` coordinates can be relative, or absolute, to the
-        current point by setting the ``relative`` flag. The coordinate then
-        becomes the new current point.
-
-        :param to: (:class:`~numbers.Real`, :class:`numbers.Real`)
-                      pair which represents coordinates to draw to.
-        :type to: :class:`collections.Sequence`
-        :param relative: :class:`bool`
-                    treat given coordinates as relative to current point
-        :type relative: :class:`bool`
-
-        .. versionadded:: 0.4.0
-
-        """
-        if to is None:
-            raise TypeError('to is missing')
-        x, y = to
-        if relative:
-            library.DrawPathLineToRelative(self.resource, float(x), float(y))
-        else:
-            library.DrawPathLineToAbsolute(self.resource, float(x), float(y))
-        return self
-
     def path_horizontal_line(self, x=None, relative=False):
         """Draws a horizontal line path from the current point to the target
         point. Given ``x`` parameter can be relative, or absolute, to the
@@ -1379,28 +1494,29 @@ class Drawing(Resource):
             library.DrawPathLineToHorizontalAbsolute(self.resource, float(x))
         return self
 
-    def path_vertical_line(self, y=None, relative=False):
-        """Draws a vertical line path from the current point to the target
-        point. Given ``y`` parameter can be relative, or absolute, to the
-        current point by setting the ``relative`` flag. The target point then
+    def path_line(self, to=None, relative=False):
+        """Draws a line path from the current point to the given ``to``
+        coordinate. The ``to`` coordinates can be relative, or absolute, to the
+        current point by setting the ``relative`` flag. The coordinate then
         becomes the new current point.
 
-        :param y: :class:`~numbers.Real`
-                      y-axis point to draw to.
-        :type y: :class:`~numbers.Real`
+        :param to: (:class:`~numbers.Real`, :class:`numbers.Real`)
+                      pair which represents coordinates to draw to.
+        :type to: :class:`collections.Sequence`
         :param relative: :class:`bool`
-                    treat given point as relative to current point
+                    treat given coordinates as relative to current point
         :type relative: :class:`bool`
 
         .. versionadded:: 0.4.0
 
         """
-        if y is None:
-            raise TypeError('y is missing')
+        if to is None:
+            raise TypeError('to is missing')
+        x, y = to
         if relative:
-            library.DrawPathLineToVerticalRelative(self.resource, float(y))
+            library.DrawPathLineToRelative(self.resource, float(x), float(y))
         else:
-            library.DrawPathLineToVerticalAbsolute(self.resource, float(y))
+            library.DrawPathLineToAbsolute(self.resource, float(x), float(y))
         return self
 
     def path_move(self, to=None, relative=False):
@@ -1440,6 +1556,72 @@ class Drawing(Resource):
         library.DrawPathStart(self.resource)
         return self
 
+    def path_vertical_line(self, y=None, relative=False):
+        """Draws a vertical line path from the current point to the target
+        point. Given ``y`` parameter can be relative, or absolute, to the
+        current point by setting the ``relative`` flag. The target point then
+        becomes the new current point.
+
+        :param y: :class:`~numbers.Real`
+                      y-axis point to draw to.
+        :type y: :class:`~numbers.Real`
+        :param relative: :class:`bool`
+                    treat given point as relative to current point
+        :type relative: :class:`bool`
+
+        .. versionadded:: 0.4.0
+
+        """
+        if y is None:
+            raise TypeError('y is missing')
+        if relative:
+            library.DrawPathLineToVerticalRelative(self.resource, float(y))
+        else:
+            library.DrawPathLineToVerticalAbsolute(self.resource, float(y))
+        return self
+
+    def polygon(self, points=None):
+        """Draws a polygon using the current :attr:`stoke_color`,
+        :attr:`stroke_width`, and :attr:`fill_color`, using the specified
+        array of coordinates.
+
+        Example polygon on ``image`` ::
+
+            with Drawing() as draw:
+                points = [(40,10), (20,50), (90,10), (70,40)]
+                draw.polygon(points)
+                draw.draw(image)
+
+        :param points: list of x,y tuples
+        :type points: :class:`list`
+
+        .. versionadded:: 0.4.0
+
+        """
+
+        (points_l, points_p) = _list_to_point_info(points)
+        library.DrawPolygon(self.resource, points_l,
+                            ctypes.cast(points_p, ctypes.POINTER(PointInfo)))
+
+    def polyline(self, points=None):
+        """Draws a polyline using the current :attr:`stoke_color`,
+        :attr:`stroke_width`, and :attr:`fill_color`, using the specified
+        array of coordinates.
+
+        Identical to :class:`~wand.drawing.Drawing.polygon`, but without closed
+        stroke line.
+
+        :param points: list of x,y tuples
+        :type points: :class:`list`
+
+        .. versionadded:: 0.4.0
+
+        """
+
+        (points_l, points_p) = _list_to_point_info(points)
+        library.DrawPolyline(self.resource, points_l,
+                             ctypes.cast(points_p, ctypes.POINTER(PointInfo)))
+
     def point(self, x, y):
         """Draws a point at given ``x`` and ``y``
 
@@ -1456,12 +1638,16 @@ class Drawing(Resource):
                           float(y))
 
     def pop(self):
-        """Pop destroys the current drawing wand and returns to the previously
-        pushed drawing wand. Multiple drawing wands may exist. It is an error
-        to attempt to pop more drawing wands than have been pushed, and it is
-        proper form to pop all drawing wands which have been pushed.
+        """Pop destroys the current tip of the drawing context stack,
+        and restores the parent style context.
+        See :meth:`push()` method for an example.
 
-        :returns: success of pop operation
+        .. note::
+
+            Popping the graphical context stack will not erase,
+            or alter, any previously executed drawing commands.
+
+        :returns: success of pop operation.
         :rtype: `bool`
 
         .. versionadded:: 0.4.0
@@ -1494,12 +1680,52 @@ class Drawing(Resource):
         library.DrawPopPattern(self.resource)
 
     def push(self):
-        """Push clones the current drawing wand to create a new drawing wand.
-        The original drawing wand(s) may be returned to by invoking
-        :class:`Drawing.pop`. The drawing wands are stored on a drawing wand
-        stack. For every Pop there must have already been an equivalent Push.
+        """Grows the current drawing context stack by one, and inherits
+        the previous style attributes. Use :class:`Drawing.pop` to return
+        to restore previous style attributes.
 
-        :returns: success of push operation
+        This is useful for drawing shapes with diffrent styles
+        without repeatedly setting the similar
+        :meth:`fill_color <wand.drawing.Drawing.fill_color>` &
+        :meth:`stroke_color <wand.drawing.Drawing.stroke_color>` properties.
+
+        For example::
+
+            with Drawing() as ctx:
+                ctx.fill_color = Color('GREEN')
+                ctx.stroke_color = Color('ORANGE')
+                ctx.push()
+                ctx.fill_color = Color('RED')
+                ctx.text(x1, y1, 'this is RED with ORANGE outline')
+                ctx.push()
+                ctx.stroke_color = Color('BLACK')
+                ctx.text(x2, y2, 'this is RED with BLACK outline')
+                ctx.pop()
+                ctx.pop()
+                ctx.text(x3, y3, 'this is GREEN with ORANGE outline')
+
+        Which translate to the following MVG::
+
+            push graphic-context
+                fill "GREEN"
+                stroke "ORANGE"
+                push graphic-context
+                    fill "RED"
+                    text x1,y1 "this is RED with ORANGE outline"
+                    push graphic-context
+                        stroke "BLACK"
+                        text x2,y2 "this is RED with BLACK outline"
+                    pop graphic-context
+                pop graphic-context
+                text x3,y3 "this is GREEN with ORANGE outline"
+            pop graphic-context
+
+        .. note::
+
+            Pushing graphical context does not reset any previously
+            drawn artifacts.
+
+        :returns: success of push operation.
         :rtype: `bool`
 
         .. versionadded:: 0.4.0
@@ -1573,7 +1799,7 @@ class Drawing(Resource):
     def rectangle(self, left=None, top=None, right=None, bottom=None,
                   width=None, height=None, radius=None, xradius=None,
                   yradius=None):
-        """Draws a rectangle using the current :attr:`stoke_color`,
+        """Draws a rectangle using the current :attr:`stroke_color`,
         :attr:`stroke_width`, and :attr:`fill_color`.
 
         .. sourcecode:: text
@@ -1699,113 +1925,6 @@ class Drawing(Resource):
         """
         library.DrawRotate(self.resource, float(degree))
 
-    def polygon(self, points=None):
-        """Draws a polygon using the current :attr:`stoke_color`,
-        :attr:`stroke_width`, and :attr:`fill_color`, using the specified
-        array of coordinates.
-
-        Example polygon on ``image`` ::
-
-            with Drawing() as draw:
-                points = [(40,10), (20,50), (90,10), (70,40)]
-                draw.polygon(points)
-                draw.draw(image)
-
-        :param points: list of x,y tuples
-        :type points: :class:`list`
-
-        .. versionadded:: 0.4.0
-
-        """
-
-        (points_l, points_p) = _list_to_point_info(points)
-        library.DrawPolygon(self.resource, points_l,
-                            ctypes.cast(points_p, ctypes.POINTER(PointInfo)))
-
-    def polyline(self, points=None):
-        """Draws a polyline using the current :attr:`stoke_color`,
-        :attr:`stroke_width`, and :attr:`fill_color`, using the specified
-        array of coordinates.
-
-        Identical to :class:`~wand.drawing.Drawing.polygon`, but without closed
-        stroke line.
-
-        :param points: list of x,y tuples
-        :type points: :class:`list`
-
-        .. versionadded:: 0.4.0
-
-        """
-
-        (points_l, points_p) = _list_to_point_info(points)
-        library.DrawPolyline(self.resource, points_l,
-                             ctypes.cast(points_p, ctypes.POINTER(PointInfo)))
-
-    def bezier(self, points=None):
-        """Draws a bezier curve through a set of points on the image, using
-        the specified array of coordinates.
-
-        At least four points should be given to complete a bezier path.
-        The first & forth point being the start & end point, and the second
-        & third point controlling the direction & curve.
-
-        Example bezier on ``image`` ::
-
-            with Drawing() as draw:
-                points = [(40,10), # Start point
-                          (20,50), # First control
-                          (90,10), # Second control
-                          (70,40)] # End point
-                draw.stroke_color = Color('#000')
-                draw.fill_color = Color('#fff')
-                draw.bezier(points)
-                draw.draw(image)
-
-        :param points: list of x,y tuples
-        :type points: :class:`list`
-
-        .. versionadded:: 0.4.0
-
-        """
-
-        (points_l, points_p) = _list_to_point_info(points)
-        library.DrawBezier(self.resource, points_l,
-                           ctypes.cast(points_p, ctypes.POINTER(PointInfo)))
-
-    def text(self, x, y, body):
-        """Writes a text ``body`` into (``x``, ``y``).
-
-        :param x: the left offset where to start writing a text
-        :type x: :class:`numbers.Integral`
-        :param y: the baseline where to start writing text
-        :type y: :class:`numbers.Integral`
-        :param body: the body string to write
-        :type body: :class:`basestring`
-
-        """
-        if not isinstance(x, numbers.Integral) or x < 0:
-            exc = ValueError if x < 0 else TypeError
-            raise exc('x must be a natural number, not ' + repr(x))
-        elif not isinstance(y, numbers.Integral) or y < 0:
-            exc = ValueError if y < 0 else TypeError
-            raise exc('y must be a natural number, not ' + repr(y))
-        elif not isinstance(body, string_type):
-            raise TypeError('body must be a string, not ' + repr(body))
-        elif not body:
-            raise ValueError('body string cannot be empty')
-        if isinstance(body, text_type):
-            # According to ImageMagick C API docs, we can use only UTF-8
-            # at this time, so we do hardcoding here.
-            # http://imagemagick.org/api/drawing-wand.php#DrawSetTextEncoding
-            if not self.text_encoding:
-                self.text_encoding = 'UTF-8'
-            body = body.encode(self.text_encoding)
-        body_p = ctypes.create_string_buffer(body)
-        library.DrawAnnotation(
-            self.resource, x, y,
-            ctypes.cast(body_p, ctypes.POINTER(ctypes.c_ubyte))
-        )
-
     def scale(self, x=None, y=None):
         """
         Adjusts the scaling factor to apply in the horizontal and vertical
@@ -1886,6 +2005,40 @@ class Drawing(Resource):
         if y is not None:
             library.DrawSkewY(self.resource, float(y))
 
+    def text(self, x, y, body):
+        """Writes a text ``body`` into (``x``, ``y``).
+
+        :param x: the left offset where to start writing a text
+        :type x: :class:`numbers.Integral`
+        :param y: the baseline where to start writing text
+        :type y: :class:`numbers.Integral`
+        :param body: the body string to write
+        :type body: :class:`basestring`
+
+        """
+        if not isinstance(x, numbers.Integral) or x < 0:
+            exc = ValueError if x < 0 else TypeError
+            raise exc('x must be a natural number, not ' + repr(x))
+        elif not isinstance(y, numbers.Integral) or y < 0:
+            exc = ValueError if y < 0 else TypeError
+            raise exc('y must be a natural number, not ' + repr(y))
+        elif not isinstance(body, string_type):
+            raise TypeError('body must be a string, not ' + repr(body))
+        elif not body:
+            raise ValueError('body string cannot be empty')
+        if isinstance(body, text_type):
+            # According to ImageMagick C API docs, we can use only UTF-8
+            # at this time, so we do hardcoding here.
+            # http://imagemagick.org/api/drawing-wand.php#DrawSetTextEncoding
+            if not self.text_encoding:
+                self.text_encoding = 'UTF-8'
+            body = body.encode(self.text_encoding)
+        body_p = ctypes.create_string_buffer(body)
+        library.DrawAnnotation(
+            self.resource, x, y,
+            ctypes.cast(body_p, ctypes.POINTER(ctypes.c_ubyte))
+        )
+
     def translate(self, x=None, y=None):
         """Applies a translation to the current coordinate system which moves
         the coordinate system origin to the specified coordinate.
@@ -1901,35 +2054,6 @@ class Drawing(Resource):
             raise TypeError('Both x & y coordinates need to be defined')
         library.DrawTranslate(self.resource, float(x), float(y))
 
-    def get_font_metrics(self, image, text, multiline=False):
-        """Queries font metrics from the given ``text``.
-
-        :param image: the image to be drawn
-        :type image: :class:`~wand.image.Image`
-        :param text: the text string for get font metrics.
-        :type text: :class:`basestring`
-        :param multiline: text is multiline or not
-        :type multiline: `boolean`
-
-        """
-        if not isinstance(image, Image):
-            raise TypeError('image must be a wand.image.Image instance, not ' +
-                            repr(image))
-        if not isinstance(text, string_type):
-            raise TypeError('text must be a string, not ' + repr(text))
-        if multiline:
-            font_metrics_f = library.MagickQueryMultilineFontMetrics
-        else:
-            font_metrics_f = library.MagickQueryFontMetrics
-        if isinstance(text, text_type):
-            if self.text_encoding:
-                text = text.encode(self.text_encoding)
-            else:
-                text = binary(text)
-        result = font_metrics_f(image.wand, self.resource, text)
-        args = (result[i] for i in xrange(13))
-        return FontMetrics(*args)
-
     def viewbox(self, left, top, right, bottom):
         """Viewbox sets the overall canvas size to be recorded with the drawing
         vector data. Usually this will be specified using the same size as the
@@ -1983,7 +2107,7 @@ def _list_to_point_info(points):
     point_info_size = point_length * tuple_size
     # Allocate sequence of memory
     point_info = (ctypes.c_double * point_info_size)()
-    for double_index in xrange(0, point_info_size):
+    for double_index in range(point_info_size):
         tuple_index = double_index // tuple_size
         tuple_offset = double_index % tuple_size
         point_info[double_index] = ctypes.c_double(
